/**
 *   wsdl-common-jbi-binding - Class(es) for supporting JBI WSDL deployments
 *
 *   Copyright (C) 2007 Gestalt, LLC. All Rights Reserved.
 *   http://www.gestalt-llc.com/
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License version 2.1 as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 */
package com.gestalt.jbi.component.manager.deployment;

import com.gestalt.jbi.component.AbstractComponent;

import com.sun.jbi.internationalization.Messages;

import org.w3c.dom.Document;

import java.io.File;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.wsdl.*;
import javax.wsdl.extensions.ExtensibilityElement;
import javax.wsdl.extensions.ExtensionRegistry;
import javax.wsdl.factory.WSDLFactory;
import javax.wsdl.xml.WSDLReader;

import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;


public abstract class AbstractWSDLDeployer extends AbstractDeployer {
    /**
     * The log for this class and subclasses.
     * <p>
     * <b>Note to developers:</b>
     * Whether or not it is good practice to use inherited loggers,
     * some subclasses of this class <i>do</i> expect to inherit this,
     * so leave it either public or protected.
     */
    protected static Logger log = Messages.getLogger(AbstractWSDLDeployer.class);
    protected static WSDLFactory wsdlFactory;
    protected static DocumentBuilderFactory documentFactory = DocumentBuilderFactory.newInstance();
    protected static DocumentBuilder documentBuilder;
    private Messages messages = Messages.getMessages(AbstractWSDLDeployer.class);

    public AbstractWSDLDeployer(AbstractComponent component) {
        super(component);

        try {
            wsdlFactory = WSDLFactory.newInstance();
            documentBuilder = documentFactory.newDocumentBuilder();
        } catch (Exception e) {
            log.log(Level.WARNING,
                messages.getString(
                    "CJBIRT-W00950.exceptionConstructingAbstractWSDLDeployer"),
                e);
        }
    }

    protected boolean isDeployable(File file) {
        return file.getName().endsWith(".wsdl");
    }

    public Definition getWsdlDefinition(File wsdl) throws WSDLException {
        WSDLReader reader = wsdlFactory.newWSDLReader();
        reader.setExtensionRegistry(createExtensionRegistry());

        return reader.readWSDL(wsdl.getAbsolutePath());
    }

    public Document getWsdlDocument(File wsdl) throws Exception {
        return documentBuilder.parse(wsdl);
    }

    protected Binding getBinding(Definition definition, String serviceName,
        String endpointName) {
        Port port = getPort(definition, serviceName, endpointName);

        if (port == null) {
            log.warning(messages.getString(
                    "CJBIRT-W00970.noPortFoundForBinding"));

            return null;
        }

        return port.getBinding();
    }

    protected Port getPort(Definition definition, String serviceName,
        String endpointName) {
        if (log.isLoggable(Level.FINER)) {
            log.log(Level.FINER, "Looking for serviceName - " + serviceName);
        }

        Service svc = definition.getService(QName.valueOf(serviceName));

        if (svc == null) {
            log.warning(messages.getString("CJBIRT-W00972.serviceNotFound",
                    serviceName));

            return null;
        }

        log.log(Level.INFO, "Looking for the endpointName - " + endpointName);

        Port port = svc.getPort(QName.valueOf(endpointName).getLocalPart());

        if (port == null) {
            log.log(Level.WARNING,
                messages.getString("CJBIRT-W00975.portNotFound", endpointName));

            return null;
        }

        return port;
    }

    /**
     * Returns the concrete binding
     */
    protected <T extends ExtensibilityElement> T getConcreteBinding(
        Definition definition, String serviceName, String endpointName,
        Class<T> bindingClass) {
        Binding binding = getBinding(definition, serviceName, endpointName);

        if (binding != null) {
            List<ExtensibilityElement> extElems = binding.getExtensibilityElements();

            if (extElems != null) {
                for (ExtensibilityElement extensibilityElement : extElems) {
                    try {
                        if (extensibilityElement.getClass()
                                                    .isAssignableFrom(bindingClass)) {
                            return (T) extensibilityElement;
                        }
                    } catch (Exception e) {
                        log.log(Level.WARNING,
                            messages.getString(
                                "CJBIRT-W00980.exceptionGettingBinding"), e);

                        return null;
                    }
                }
            }
        }

        return null;
    }

    /**
     * Returns the concrete address
     * Want to use it like
     * RSSAddress address = getConcreteAddress(def, sn, en, RSSAddress.class)
     * UDDIAddress address = getConcreteAddress(def, sn, en, UDDIAddress.class)
     */
    protected <T extends ExtensibilityElement> T getConcreteAddress(
        Definition definition, String serviceName, String endpointName,
        Class<T> compareTo) {
        Port port = getPort(definition, serviceName, endpointName);

        if (port != null) {
            List<ExtensibilityElement> extensibilityElements = port.getExtensibilityElements();

            if (extensibilityElements != null) {
                for (ExtensibilityElement extensibilityElement : extensibilityElements) {
                    if (extensibilityElement != null) {
                        try {
                            if (extensibilityElement.getClass()
                                                        .isInstance(compareTo.newInstance())) {
                                return (T) extensibilityElement;
                            }
                        } catch (Exception e) {
                            log.log(Level.WARNING,
                                messages.getString(
                                    "CJBIRT-W00985.exceptionGettingAddress"), e);

                            return null;
                        }
                    }
                }
            }
        }

        return null;
    }

    /**
     * Creates the Concrete ExtensionRegistry class used when
     * building the WSDL Factory to register wsdl extensions.
     * @return The concrete ExtensionRegistry such as a
     * <code>new RSSExtensionRegistry()</code>
     */
    public abstract ExtensionRegistry createExtensionRegistry();
}
